# This file was auto-generated by Fern from our API Definition.

import typing

from .code_splitter import CodeSplitter
from .entity_extractor import EntityExtractor
from .hierarchical_node_parser import HierarchicalNodeParser
from .html_node_parser import HtmlNodeParser
from .hugging_face_embedding import HuggingFaceEmbedding
from .json_node_parser import JsonNodeParser
from .keyword_extractor import KeywordExtractor
from .markdown_node_parser import MarkdownNodeParser
from .marvin_metadata_extractor import MarvinMetadataExtractor
from .open_ai_embedding import OpenAiEmbedding
from .questions_answered_extractor import QuestionsAnsweredExtractor
from .sentence_splitter import SentenceSplitter
from .sentence_window_node_parser import SentenceWindowNodeParser
from .simple_file_node_parser import SimpleFileNodeParser
from .summary_extractor import SummaryExtractor
from .title_extractor import TitleExtractor
from .token_text_splitter import TokenTextSplitter

ConfiguredTransformationItemComponentOne = typing.Union[
    KeywordExtractor,
    TitleExtractor,
    EntityExtractor,
    MarvinMetadataExtractor,
    SummaryExtractor,
    QuestionsAnsweredExtractor,
    SentenceWindowNodeParser,
    HierarchicalNodeParser,
    CodeSplitter,
    SentenceSplitter,
    TokenTextSplitter,
    HtmlNodeParser,
    MarkdownNodeParser,
    JsonNodeParser,
    SimpleFileNodeParser,
    OpenAiEmbedding,
    HuggingFaceEmbedding,
]



# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

import pydantic

from llama_index_client.core.api_error import ApiError
from llama_index_client.core.client_wrapper import (
    AsyncClientWrapper,
    SyncClientWrapper,
)
from llama_index_client.types.configurable_transformation_definition import (
    ConfigurableTransformationDefinition,
)
from llama_index_client.types.data_sink_definition import DataSinkDefinition
from llama_index_client.types.data_source_definition import (
    DataSourceDefinition,
)


class ComponentDefinitionClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_all_transformation_definitions(
        self,
    ) -> typing.List[ConfigurableTransformationDefinition]:
        """
        Get all configurable transformation component definitions.
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                "api/component_definition/configurable_transformation",
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.List[ConfigurableTransformationDefinition], _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_all_data_source_definitions(self) -> typing.List[DataSourceDefinition]:
        """
        Get all data source component definitions.
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                "api/component_definition/data_source",
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.List[DataSourceDefinition], _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_all_data_sink_definitions(self) -> typing.List[DataSinkDefinition]:
        """
        Get all data sink component definitions.
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                "api/component_definition/data_sink",
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.List[DataSinkDefinition], _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncComponentDefinitionClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_all_transformation_definitions(
        self,
    ) -> typing.List[ConfigurableTransformationDefinition]:
        """
        Get all configurable transformation component definitions.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                "api/component_definition/configurable_transformation",
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.List[ConfigurableTransformationDefinition], _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_all_data_source_definitions(
        self,
    ) -> typing.List[DataSourceDefinition]:
        """
        Get all data source component definitions.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                "api/component_definition/data_source",
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.List[DataSourceDefinition], _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_all_data_sink_definitions(self) -> typing.List[DataSinkDefinition]:
        """
        Get all data sink component definitions.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                "api/component_definition/data_sink",
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.List[DataSinkDefinition], _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic

from ..core.datetime_utils import serialize_datetime


class QdrantVectorStore(pydantic.BaseModel):
    """
    Qdrant Vector Store.

    In this vector store, embeddings and docs are stored within a
    Qdrant collection.

    During query time, the index uses Qdrant to query for the top
    k most similar nodes.

    Args:
        collection_name: (str): name of the Qdrant collection
        client (Optional[Any]): QdrantClient instance from `qdrant-client` package
    """

    stores_text: typing.Optional[bool]
    is_embedding_query: typing.Optional[bool]
    flat_metadata: typing.Optional[bool]
    collection_name: str
    url: typing.Optional[str]
    api_key: typing.Optional[str]
    batch_size: int
    prefer_grpc: bool
    client_kwargs: typing.Optional[typing.Dict[str, typing.Any]]
    class_name: typing.Optional[str]

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {
            "by_alias": True,
            "exclude_unset": True,
            **kwargs,
        }
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {
            "by_alias": True,
            "exclude_unset": True,
            **kwargs,
        }
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import typing

from .data_source_component_one import DataSourceComponentOne

DataSourceComponent = typing.Union[typing.Dict[str, typing.Any], DataSourceComponentOne]



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ConfigurableDataSourceNames(str, enum.Enum):
    """
    An enumeration.
    """

    DOCUMENT = "DOCUMENT"
    TEXT_NODE = "TEXT_NODE"
    DISCORD = "DISCORD"
    ELASTICSEARCH = "ELASTICSEARCH"
    NOTION_PAGE = "NOTION_PAGE"
    SLACK = "SLACK"
    TWITTER = "TWITTER"
    SIMPLE_WEB_PAGE = "SIMPLE_WEB_PAGE"
    TRAFILATURA_WEB_PAGE = "TRAFILATURA_WEB_PAGE"
    BEAUTIFUL_SOUP_WEB_PAGE = "BEAUTIFUL_SOUP_WEB_PAGE"
    RSS = "RSS"
    WIKIPEDIA = "WIKIPEDIA"
    YOUTUBE_TRANSCRIPT = "YOUTUBE_TRANSCRIPT"
    GOOGLE_DOCS = "GOOGLE_DOCS"
    GOOGLE_SHEETS = "GOOGLE_SHEETS"
    READER = "READER"

    def visit(
        self,
        document: typing.Callable[[], T_Result],
        text_node: typing.Callable[[], T_Result],
        discord: typing.Callable[[], T_Result],
        elasticsearch: typing.Callable[[], T_Result],
        notion_page: typing.Callable[[], T_Result],
        slack: typing.Callable[[], T_Result],
        twitter: typing.Callable[[], T_Result],
        simple_web_page: typing.Callable[[], T_Result],
        trafilatura_web_page: typing.Callable[[], T_Result],
        beautiful_soup_web_page: typing.Callable[[], T_Result],
        rss: typing.Callable[[], T_Result],
        wikipedia: typing.Callable[[], T_Result],
        youtube_transcript: typing.Callable[[], T_Result],
        google_docs: typing.Callable[[], T_Result],
        google_sheets: typing.Callable[[], T_Result],
        reader: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is ConfigurableDataSourceNames.DOCUMENT:
            return document()
        if self is ConfigurableDataSourceNames.TEXT_NODE:
            return text_node()
        if self is ConfigurableDataSourceNames.DISCORD:
            return discord()
        if self is ConfigurableDataSourceNames.ELASTICSEARCH:
            return elasticsearch()
        if self is ConfigurableDataSourceNames.NOTION_PAGE:
            return notion_page()
        if self is ConfigurableDataSourceNames.SLACK:
            return slack()
        if self is ConfigurableDataSourceNames.TWITTER:
            return twitter()
        if self is ConfigurableDataSourceNames.SIMPLE_WEB_PAGE:
            return simple_web_page()
        if self is ConfigurableDataSourceNames.TRAFILATURA_WEB_PAGE:
            return trafilatura_web_page()
        if self is ConfigurableDataSourceNames.BEAUTIFUL_SOUP_WEB_PAGE:
            return beautiful_soup_web_page()
        if self is ConfigurableDataSourceNames.RSS:
            return rss()
        if self is ConfigurableDataSourceNames.WIKIPEDIA:
            return wikipedia()
        if self is ConfigurableDataSourceNames.YOUTUBE_TRANSCRIPT:
            return youtube_transcript()
        if self is ConfigurableDataSourceNames.GOOGLE_DOCS:
            return google_docs()
        if self is ConfigurableDataSourceNames.GOOGLE_SHEETS:
            return google_sheets()
        if self is ConfigurableDataSourceNames.READER:
            return reader()



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic

from ..core.datetime_utils import serialize_datetime


class OpenAiEmbedding(pydantic.BaseModel):
    """
    OpenAI class for embeddings.

    Args:
        mode (str): Mode for embedding.
            Defaults to OpenAIEmbeddingMode.TEXT_SEARCH_MODE.
            Options are:

            - OpenAIEmbeddingMode.SIMILARITY_MODE
            - OpenAIEmbeddingMode.TEXT_SEARCH_MODE

        model (str): Model for embedding.
            Defaults to OpenAIEmbeddingModelType.TEXT_EMBED_ADA_002.
            Options are:

            - OpenAIEmbeddingModelType.DAVINCI
            - OpenAIEmbeddingModelType.CURIE
            - OpenAIEmbeddingModelType.BABBAGE
            - OpenAIEmbeddingModelType.ADA
            - OpenAIEmbeddingModelType.TEXT_EMBED_ADA_002
    """

    model_name: typing.Optional[str] = pydantic.Field(
        description="The name of the embedding model."
    )
    embed_batch_size: typing.Optional[int] = pydantic.Field(
        description="The batch size for embedding calls."
    )
    callback_manager: typing.Optional[typing.Dict[str, typing.Any]]
    additional_kwargs: typing.Optional[typing.Dict[str, typing.Any]] = pydantic.Field(
        description="Additional kwargs for the OpenAI API."
    )
    api_key: str = pydantic.Field(description="The OpenAI API key.")
    api_base: str = pydantic.Field(description="The base URL for OpenAI API.")
    api_version: str = pydantic.Field(description="The version for OpenAI API.")
    max_retries: typing.Optional[int] = pydantic.Field(
        description="Maximum number of retries."
    )
    class_name: typing.Optional[str]

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {
            "by_alias": True,
            "exclude_unset": True,
            **kwargs,
        }
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {
            "by_alias": True,
            "exclude_unset": True,
            **kwargs,
        }
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic

from ..core.datetime_utils import serialize_datetime
from .etl_job_names import EtlJobNames
from .eval_dataset_execution_run_config_per_op_value import (
    EvalDatasetExecutionRunConfigPerOpValue,
)
from .eval_execution_params import EvalExecutionParams
from .status_enum import StatusEnum


class EvalDatasetExecution(pydantic.BaseModel):
    """
    Schema for job that evaluates an EvalDataset against a pipeline.
    """

    id: typing.Optional[str] = pydantic.Field(description="Unique identifier")
    created_at: typing.Optional[dt.datetime] = pydantic.Field(
        description="Creation datetime"
    )
    updated_at: typing.Optional[dt.datetime] = pydantic.Field(
        description="Update datetime"
    )
    job_name: EtlJobNames
    status: StatusEnum
    started_at: typing.Optional[dt.datetime]
    ended_at: typing.Optional[dt.datetime]
    run_config_per_op: typing.Dict[
        str, EvalDatasetExecutionRunConfigPerOpValue
    ] = pydantic.Field(description="Run config that was given to the Dagster job")
    pipeline_id: str = pydantic.Field(
        description="The ID for the Pipeline this execution ran against."
    )
    eval_dataset_id: str = pydantic.Field(
        description="The ID for the EvalDataset this execution ran against."
    )
    eval_question_ids: typing.List[str] = pydantic.Field(
        description="The IDs for the EvalQuestions this execution ran against."
    )
    eval_execution_params: EvalExecutionParams = pydantic.Field(
        description="The parameters for the eval execution."
    )

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {
            "by_alias": True,
            "exclude_unset": True,
            **kwargs,
        }
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {
            "by_alias": True,
            "exclude_unset": True,
            **kwargs,
        }
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class Pooling(str, enum.Enum):
    """
    Enum of possible pooling choices with pooling behaviors.
    """

    CLS = "cls"
    MEAN = "mean"

    def visit(
        self, cls: typing.Callable[[], T_Result], mean: typing.Callable[[], T_Result]
    ) -> T_Result:
        if self is Pooling.CLS:
            return cls()
        if self is Pooling.MEAN:
            return mean()



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic

from ..core.datetime_utils import serialize_datetime
from .node_parser import NodeParser


class HierarchicalNodeParser(pydantic.BaseModel):
    """
    Hierarchical node parser.

    Splits a document into a recursive hierarchy Nodes using a NodeParser.

    NOTE: this will return a hierarchy of nodes in a flat list, where there will be
    overlap between parent nodes (e.g. with a bigger chunk size), and child nodes
    per parent (e.g. with a smaller chunk size).

    For instance, this may return a list of nodes like:
    - list of top-level nodes with chunk size 2048
    - list of second-level nodes, where each node is a child of a top-level node,
        chunk size 512
    - list of third-level nodes, where each node is a child of a second-level node,
        chunk size 128
    """

    include_metadata: typing.Optional[bool] = pydantic.Field(
        description="Whether or not to consider metadata when splitting."
    )
    include_prev_next_rel: typing.Optional[bool] = pydantic.Field(
        description="Include prev/next node relationships."
    )
    callback_manager: typing.Optional[typing.Dict[str, typing.Any]]
    chunk_sizes: typing.Optional[typing.List[int]] = pydantic.Field(
        description="The chunk sizes to use when splitting documents, in order of level."
    )
    node_parser_ids: typing.Optional[typing.List[str]] = pydantic.Field(
        description="List of ids for the node parsers to use when splitting documents, in order of level (first id used for first level, etc.)."
    )
    node_parser_map: typing.Dict[str, NodeParser] = pydantic.Field(
        description="Map of node parser id to node parser."
    )
    class_name: typing.Optional[str]

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {
            "by_alias": True,
            "exclude_unset": True,
            **kwargs,
        }
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {
            "by_alias": True,
            "exclude_unset": True,
            **kwargs,
        }
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ObjectType(str, enum.Enum):
    """
    An enumeration.
    """

    ONE = "1"
    TWO = "2"
    THREE = "3"
    FOUR = "4"

    def visit(
        self,
        one: typing.Callable[[], T_Result],
        two: typing.Callable[[], T_Result],
        three: typing.Callable[[], T_Result],
        four: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is ObjectType.ONE:
            return one()
        if self is ObjectType.TWO:
            return two()
        if self is ObjectType.THREE:
            return three()
        if self is ObjectType.FOUR:
            return four()



# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

import pydantic

from llama_index_client.core.api_error import ApiError
from llama_index_client.core.client_wrapper import (
    AsyncClientWrapper,
    SyncClientWrapper,
)
from llama_index_client.core.jsonable_encoder import jsonable_encoder
from llama_index_client.errors.unprocessable_entity_error import (
    UnprocessableEntityError,
)
from llama_index_client.types.configurable_data_source_names import (
    ConfigurableDataSourceNames,
)
from llama_index_client.types.data_source import DataSource
from llama_index_client.types.data_source_create import DataSourceCreate
from llama_index_client.types.data_source_load_execution import (
    DataSourceLoadExecution,
)
from llama_index_client.types.data_source_update_component import (
    DataSourceUpdateComponent,
)
from llama_index_client.types.document import Document
from llama_index_client.types.http_validation_error import HttpValidationError

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class DataSourceClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create_data_source(self, *, request: DataSourceCreate) -> DataSource:
        """
        Create a new data source.

        Parameters:
            - request: DataSourceCreate.
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", "api/data_source"
            ),
            json=jsonable_encoder(request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(DataSource, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def upsert_data_source(self, *, request: DataSourceCreate) -> DataSource:
        """
        Upserts a data source.
        Updates if a data source with the same name and user_id already exists.Otherwise, creates a new data source.

        Parameters:
            - request: DataSourceCreate.
        """
        _response = self._client_wrapper.httpx_client.request(
            "PUT",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", "api/data_source"
            ),
            json=jsonable_encoder(request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(DataSource, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_data_source(self, data_source_id: str) -> DataSource:
        """
        Get a data source by ID.

        Parameters:
            - data_source_id: str.
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"api/data_source/{data_source_id}",
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(DataSource, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update_data_source(
        self,
        data_source_id: str,
        *,
        source_type: ConfigurableDataSourceNames,
        component: typing.Optional[DataSourceUpdateComponent] = OMIT,
        name: typing.Optional[str] = OMIT,
    ) -> DataSource:
        """
        Update a data source by ID.

        Parameters:
            - data_source_id: str.

            - source_type: ConfigurableDataSourceNames.

            - component: typing.Optional[DataSourceUpdateComponent].

            - name: typing.Optional[str].
        """
        _request: typing.Dict[str, typing.Any] = {"source_type": source_type}
        if component is not OMIT:
            _request["component"] = component
        if name is not OMIT:
            _request["name"] = name
        _response = self._client_wrapper.httpx_client.request(
            "PUT",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"api/data_source/{data_source_id}",
            ),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(DataSource, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete_data_source(self, data_source_id: str) -> None:
        """
        Delete a data source by ID.

        Parameters:
            - data_source_id: str.
        """
        _response = self._client_wrapper.httpx_client.request(
            "DELETE",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"api/data_source/{data_source_id}",
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_documents_metadata(self, data_source_id: str) -> typing.List[Document]:
        """
        Get content and metadata for documents by data source ID.

        Parameters:
            - data_source_id: str.
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"api/data_source/{data_source_id}/documents",
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.List[Document], _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_data_source_executions(
        self, data_source_id: str
    ) -> typing.List[DataSourceLoadExecution]:
        """
        Get all data source executions for a given data source.

        Parameters:
            - data_source_id: str.
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"api/data_source/{data_source_id}/execution",
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.List[DataSourceLoadExecution], _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create_data_source_execution(
        self, data_source_id: str
    ) -> DataSourceLoadExecution:
        """
        Create a new data source execution.

        Parameters:
            - data_source_id: str.
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"api/data_source/{data_source_id}/execution",
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(DataSourceLoadExecution, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_data_source_execution(
        self, data_source_id: str, data_source_load_execution_id: str
    ) -> DataSourceLoadExecution:
        """
        Get a data source execution by ID.

        Parameters:
            - data_source_id: str.

            - data_source_load_execution_id: str.
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"api/data_source/{data_source_id}/execution/{data_source_load_execution_id}",
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(DataSourceLoadExecution, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncDataSourceClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create_data_source(self, *, request: DataSourceCreate) -> DataSource:
        """
        Create a new data source.

        Parameters:
            - request: DataSourceCreate.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", "api/data_source"
            ),
            json=jsonable_encoder(request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(DataSource, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def upsert_data_source(self, *, request: DataSourceCreate) -> DataSource:
        """
        Upserts a data source.
        Updates if a data source with the same name and user_id already exists.Otherwise, creates a new data source.

        Parameters:
            - request: DataSourceCreate.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "PUT",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", "api/data_source"
            ),
            json=jsonable_encoder(request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(DataSource, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_data_source(self, data_source_id: str) -> DataSource:
        """
        Get a data source by ID.

        Parameters:
            - data_source_id: str.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"api/data_source/{data_source_id}",
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(DataSource, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update_data_source(
        self,
        data_source_id: str,
        *,
        source_type: ConfigurableDataSourceNames,
        component: typing.Optional[DataSourceUpdateComponent] = OMIT,
        name: typing.Optional[str] = OMIT,
    ) -> DataSource:
        """
        Update a data source by ID.

        Parameters:
            - data_source_id: str.

            - source_type: ConfigurableDataSourceNames.

            - component: typing.Optional[DataSourceUpdateComponent].

            - name: typing.Optional[str].
        """
        _request: typing.Dict[str, typing.Any] = {"source_type": source_type}
        if component is not OMIT:
            _request["component"] = component
        if name is not OMIT:
            _request["name"] = name
        _response = await self._client_wrapper.httpx_client.request(
            "PUT",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"api/data_source/{data_source_id}",
            ),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(DataSource, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete_data_source(self, data_source_id: str) -> None:
        """
        Delete a data source by ID.

        Parameters:
            - data_source_id: str.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "DELETE",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"api/data_source/{data_source_id}",
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_documents_metadata(
        self, data_source_id: str
    ) -> typing.List[Document]:
        """
        Get content and metadata for documents by data source ID.

        Parameters:
            - data_source_id: str.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"api/data_source/{data_source_id}/documents",
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.List[Document], _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_data_source_executions(
        self, data_source_id: str
    ) -> typing.List[DataSourceLoadExecution]:
        """
        Get all data source executions for a given data source.

        Parameters:
            - data_source_id: str.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"api/data_source/{data_source_id}/execution",
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.List[DataSourceLoadExecution], _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create_data_source_execution(
        self, data_source_id: str
    ) -> DataSourceLoadExecution:
        """
        Create a new data source execution.

        Parameters:
            - data_source_id: str.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"api/data_source/{data_source_id}/execution",
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(DataSourceLoadExecution, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_data_source_execution(
        self, data_source_id: str, data_source_load_execution_id: str
    ) -> DataSourceLoadExecution:
        """
        Get a data source execution by ID.

        Parameters:
            - data_source_id: str.

            - data_source_load_execution_id: str.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"api/data_source/{data_source_id}/execution/{data_source_load_execution_id}",
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(DataSourceLoadExecution, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

from .beautiful_soup_web_reader import BeautifulSoupWebReader
from .discord_reader import DiscordReader
from .document import Document
from .elasticsearch_reader import ElasticsearchReader
from .google_docs_reader import GoogleDocsReader
from .google_sheets_reader import GoogleSheetsReader
from .notion_page_reader import NotionPageReader
from .reader_config import ReaderConfig
from .rss_reader import RssReader
from .simple_web_page_reader import SimpleWebPageReader
from .slack_reader import SlackReader
from .text_node import TextNode
from .trafilatura_web_reader import TrafilaturaWebReader
from .twitter_tweet_reader import TwitterTweetReader
from .wikipedia_reader import WikipediaReader
from .youtube_transcript_reader import YoutubeTranscriptReader

DataSourceUpdateComponentOne = typing.Union[
    Document,
    TextNode,
    DiscordReader,
    ElasticsearchReader,
    NotionPageReader,
    SlackReader,
    TwitterTweetReader,
    SimpleWebPageReader,
    TrafilaturaWebReader,
    BeautifulSoupWebReader,
    RssReader,
    WikipediaReader,
    YoutubeTranscriptReader,
    GoogleDocsReader,
    GoogleSheetsReader,
    ReaderConfig,
]



# This file was auto-generated by Fern from our API Definition.

import typing

from .eval_dataset_execution_config import EvalDatasetExecutionConfig
from .run_transform_asset_config import RunTransformAssetConfig

EvalDatasetExecutionRunConfigPerOpValue = typing.Union[
    RunTransformAssetConfig, EvalDatasetExecutionConfig
]



# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

import pydantic

from llama_index_client.core.api_error import ApiError
from llama_index_client.core.client_wrapper import (
    AsyncClientWrapper,
    SyncClientWrapper,
)
from llama_index_client.core.jsonable_encoder import jsonable_encoder
from llama_index_client.core.remove_none_from_dict import (
    remove_none_from_dict,
)
from llama_index_client.errors.unprocessable_entity_error import (
    UnprocessableEntityError,
)
from llama_index_client.types.configured_transformation_execution import (
    ConfiguredTransformationExecution,
)
from llama_index_client.types.configured_transformation_item import (
    ConfiguredTransformationItem,
)
from llama_index_client.types.data_sink_create import DataSinkCreate
from llama_index_client.types.data_source_create import DataSourceCreate
from llama_index_client.types.eval_dataset_execution import (
    EvalDatasetExecution,
)
from llama_index_client.types.eval_execution_params import EvalExecutionParams
from llama_index_client.types.eval_question_result import EvalQuestionResult
from llama_index_client.types.http_validation_error import HttpValidationError
from llama_index_client.types.pipeline import Pipeline
from llama_index_client.types.text_node import TextNode

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class PipelineClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_pipeline_by_name(
        self, *, pipeline_name: str, project_name: str
    ) -> typing.List[Pipeline]:
        """
        Get a pipeline by name.

        Parameters:
            - pipeline_name: str.

            - project_name: str.
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", "api/pipeline"
            ),
            params=remove_none_from_dict(
                {"pipeline_name": pipeline_name, "project_name": project_name}
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.List[Pipeline], _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_pipeline_for_project(self, pipeline_id: str) -> Pipeline:
        """
        Get a pipeline by ID for a given project.

        Parameters:
            - pipeline_id: str.
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"api/pipeline/{pipeline_id}"
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Pipeline, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update_existing_pipeline(
        self,
        pipeline_id: str,
        *,
        configured_transformations: typing.Optional[
            typing.List[ConfiguredTransformationItem]
        ] = OMIT,
        data_source_ids: typing.Optional[typing.List[str]] = OMIT,
        data_sources: typing.Optional[typing.List[DataSourceCreate]] = OMIT,
        data_sink_ids: typing.Optional[typing.List[str]] = OMIT,
        data_sinks: typing.Optional[typing.List[DataSinkCreate]] = OMIT,
        name: typing.Optional[str] = OMIT,
    ) -> Pipeline:
        """
        Update an existing pipeline for a project.

        Parameters:
            - pipeline_id: str.

            - configured_transformations: typing.Optional[typing.List[ConfiguredTransformationItem]].

            - data_source_ids: typing.Optional[typing.List[str]]. List of data source IDs. When provided instead of data_sources, the data sources will be looked up by ID.

            - data_sources: typing.Optional[typing.List[DataSourceCreate]]. List of data sources. When provided instead of data_source_ids, the data sources will be created.

            - data_sink_ids: typing.Optional[typing.List[str]]. List of data sink IDs. When provided instead of data_sinks, the data sinks will be looked up by ID.

            - data_sinks: typing.Optional[typing.List[DataSinkCreate]]. List of data sinks. When provided instead of data_sink_ids, the data sinks will be created.

            - name: typing.Optional[str].
        """
        _request: typing.Dict[str, typing.Any] = {}
        if configured_transformations is not OMIT:
            _request["configured_transformations"] = configured_transformations
        if data_source_ids is not OMIT:
            _request["data_source_ids"] = data_source_ids
        if data_sources is not OMIT:
            _request["data_sources"] = data_sources
        if data_sink_ids is not OMIT:
            _request["data_sink_ids"] = data_sink_ids
        if data_sinks is not OMIT:
            _request["data_sinks"] = data_sinks
        if name is not OMIT:
            _request["name"] = name
        _response = self._client_wrapper.httpx_client.request(
            "PUT",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"api/pipeline/{pipeline_id}"
            ),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Pipeline, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_all_configured_transformation_executions(
        self, pipeline_id: str
    ) -> typing.List[ConfiguredTransformationExecution]:
        """
        Get all ConfiguredTransformationExecutions for a given pipeline.

        Parameters:
            - pipeline_id: str.
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"api/pipeline/{pipeline_id}/configured_transformation_execution",
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.List[ConfiguredTransformationExecution], _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create_configured_transformation_execution(
        self,
        pipeline_id: str,
        *,
        configured_transformation_execution_id: typing.Optional[str] = None,
    ) -> ConfiguredTransformationExecution:
        """
        Kick off a new ConfiguredTransformation execution.
        Can optionally supply a configured_transformation_execution_id to run a specific execution.
        In absence of a configured_transformation_execution_id, the last
        configured_transformation_execution will be run (which may end up triggering runs
        for it's prior steps as well)

        Parameters:
            - pipeline_id: str.

            - configured_transformation_execution_id: typing.Optional[str].
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"api/pipeline/{pipeline_id}/configured_transformation_execution",
            ),
            params=remove_none_from_dict(
                {
                    "configured_transformation_execution_id": configured_transformation_execution_id
                }
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ConfiguredTransformationExecution, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_configured_transformation_execution(
        self, pipeline_id: str, configured_transformation_execution_id: str
    ) -> ConfiguredTransformationExecution:
        """
        Get status of a single pipeline ConfiguredTransformationExecution for a given pipeline.

        Parameters:
            - pipeline_id: str.

            - configured_transformation_execution_id: str.
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"api/pipeline/{pipeline_id}/configured_transformation_execution/{configured_transformation_execution_id}",
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ConfiguredTransformationExecution, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_configured_transformation_result(
        self, pipeline_id: str, configured_transformation_id: str
    ) -> typing.List[TextNode]:
        """
        Get the result of an ConfiguredTransformationExecution step.
        Unlike get_configured_transformation_execution_result, this endpoint does
        not check the status of the execution that produced the result for the
        configured_transformation.

        Parameters:
            - pipeline_id: str.

            - configured_transformation_id: str.
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"api/pipeline/{pipeline_id}/configured_transformation/{configured_transformation_id}/result",
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.List[TextNode], _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_eval_dataset_executions(
        self, pipeline_id: str, eval_dataset_id: str
    ) -> typing.List[EvalDatasetExecution]:
        """
        Get the status of an EvalDatasetExecution.

        Parameters:
            - pipeline_id: str.

            - eval_dataset_id: str.
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"api/pipeline/{pipeline_id}/eval_dataset/{eval_dataset_id}/execute",
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.List[EvalDatasetExecution], _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def execute_eval_dataset(
        self,
        pipeline_id: str,
        eval_dataset_id: str,
        *,
        eval_question_ids: typing.List[str],
        params: EvalExecutionParams,
    ) -> EvalDatasetExecution:
        """
        Execute a dataset.

        Parameters:
            - pipeline_id: str.

            - eval_dataset_id: str.

            - eval_question_ids: typing.List[str].

            - params: EvalExecutionParams.
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"api/pipeline/{pipeline_id}/eval_dataset/{eval_dataset_id}/execute",
            ),
            json=jsonable_encoder(
                {"eval_question_ids": eval_question_ids, "params": params}
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(EvalDatasetExecution, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_eval_dataset_execution(
        self, pipeline_id: str, eval_dataset_id: str, eval_dataset_execution_id: str
    ) -> EvalDatasetExecution:
        """
        Get the status of an EvalDatasetExecution.

        Parameters:
            - pipeline_id: str.

            - eval_dataset_id: str.

            - eval_dataset_execution_id: str.
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"api/pipeline/{pipeline_id}/eval_dataset/{eval_dataset_id}/execute/{eval_dataset_execution_id}",
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(EvalDatasetExecution, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_eval_dataset_execution_result(
        self, pipeline_id: str, eval_dataset_id: str
    ) -> typing.List[EvalQuestionResult]:
        """
        Get the result of an EvalDatasetExecution.

        Parameters:
            - pipeline_id: str.

            - eval_dataset_id: str.
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"api/pipeline/{pipeline_id}/eval_dataset/{eval_dataset_id}/execute/result",
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.List[EvalQuestionResult], _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncPipelineClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_pipeline_by_name(
        self, *, pipeline_name: str, project_name: str
    ) -> typing.List[Pipeline]:
        """
        Get a pipeline by name.

        Parameters:
            - pipeline_name: str.

            - project_name: str.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", "api/pipeline"
            ),
            params=remove_none_from_dict(
                {"pipeline_name": pipeline_name, "project_name": project_name}
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.List[Pipeline], _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_pipeline_for_project(self, pipeline_id: str) -> Pipeline:
        """
        Get a pipeline by ID for a given project.

        Parameters:
            - pipeline_id: str.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"api/pipeline/{pipeline_id}"
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Pipeline, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update_existing_pipeline(
        self,
        pipeline_id: str,
        *,
        configured_transformations: typing.Optional[
            typing.List[ConfiguredTransformationItem]
        ] = OMIT,
        data_source_ids: typing.Optional[typing.List[str]] = OMIT,
        data_sources: typing.Optional[typing.List[DataSourceCreate]] = OMIT,
        data_sink_ids: typing.Optional[typing.List[str]] = OMIT,
        data_sinks: typing.Optional[typing.List[DataSinkCreate]] = OMIT,
        name: typing.Optional[str] = OMIT,
    ) -> Pipeline:
        """
        Update an existing pipeline for a project.

        Parameters:
            - pipeline_id: str.

            - configured_transformations: typing.Optional[typing.List[ConfiguredTransformationItem]].

            - data_source_ids: typing.Optional[typing.List[str]]. List of data source IDs. When provided instead of data_sources, the data sources will be looked up by ID.

            - data_sources: typing.Optional[typing.List[DataSourceCreate]]. List of data sources. When provided instead of data_source_ids, the data sources will be created.

            - data_sink_ids: typing.Optional[typing.List[str]]. List of data sink IDs. When provided instead of data_sinks, the data sinks will be looked up by ID.

            - data_sinks: typing.Optional[typing.List[DataSinkCreate]]. List of data sinks. When provided instead of data_sink_ids, the data sinks will be created.

            - name: typing.Optional[str].
        """
        _request: typing.Dict[str, typing.Any] = {}
        if configured_transformations is not OMIT:
            _request["configured_transformations"] = configured_transformations
        if data_source_ids is not OMIT:
            _request["data_source_ids"] = data_source_ids
        if data_sources is not OMIT:
            _request["data_sources"] = data_sources
        if data_sink_ids is not OMIT:
            _request["data_sink_ids"] = data_sink_ids
        if data_sinks is not OMIT:
            _request["data_sinks"] = data_sinks
        if name is not OMIT:
            _request["name"] = name
        _response = await self._client_wrapper.httpx_client.request(
            "PUT",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"api/pipeline/{pipeline_id}"
            ),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Pipeline, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_all_configured_transformation_executions(
        self, pipeline_id: str
    ) -> typing.List[ConfiguredTransformationExecution]:
        """
        Get all ConfiguredTransformationExecutions for a given pipeline.

        Parameters:
            - pipeline_id: str.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"api/pipeline/{pipeline_id}/configured_transformation_execution",
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.List[ConfiguredTransformationExecution], _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create_configured_transformation_execution(
        self,
        pipeline_id: str,
        *,
        configured_transformation_execution_id: typing.Optional[str] = None,
    ) -> ConfiguredTransformationExecution:
        """
        Kick off a new ConfiguredTransformation execution.
        Can optionally supply a configured_transformation_execution_id to run a specific execution.
        In absence of a configured_transformation_execution_id, the last
        configured_transformation_execution will be run (which may end up triggering runs
        for it's prior steps as well)

        Parameters:
            - pipeline_id: str.

            - configured_transformation_execution_id: typing.Optional[str].
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"api/pipeline/{pipeline_id}/configured_transformation_execution",
            ),
            params=remove_none_from_dict(
                {
                    "configured_transformation_execution_id": configured_transformation_execution_id
                }
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ConfiguredTransformationExecution, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_configured_transformation_execution(
        self, pipeline_id: str, configured_transformation_execution_id: str
    ) -> ConfiguredTransformationExecution:
        """
        Get status of a single pipeline ConfiguredTransformationExecution for a given pipeline.

        Parameters:
            - pipeline_id: str.

            - configured_transformation_execution_id: str.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"api/pipeline/{pipeline_id}/configured_transformation_execution/{configured_transformation_execution_id}",
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ConfiguredTransformationExecution, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_configured_transformation_result(
        self, pipeline_id: str, configured_transformation_id: str
    ) -> typing.List[TextNode]:
        """
        Get the result of an ConfiguredTransformationExecution step.
        Unlike get_configured_transformation_execution_result, this endpoint does
        not check the status of the execution that produced the result for the
        configured_transformation.

        Parameters:
            - pipeline_id: str.

            - configured_transformation_id: str.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"api/pipeline/{pipeline_id}/configured_transformation/{configured_transformation_id}/result",
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.List[TextNode], _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_eval_dataset_executions(
        self, pipeline_id: str, eval_dataset_id: str
    ) -> typing.List[EvalDatasetExecution]:
        """
        Get the status of an EvalDatasetExecution.

        Parameters:
            - pipeline_id: str.

            - eval_dataset_id: str.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"api/pipeline/{pipeline_id}/eval_dataset/{eval_dataset_id}/execute",
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.List[EvalDatasetExecution], _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def execute_eval_dataset(
        self,
        pipeline_id: str,
        eval_dataset_id: str,
        *,
        eval_question_ids: typing.List[str],
        params: EvalExecutionParams,
    ) -> EvalDatasetExecution:
        """
        Execute a dataset.

        Parameters:
            - pipeline_id: str.

            - eval_dataset_id: str.

            - eval_question_ids: typing.List[str].

            - params: EvalExecutionParams.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"api/pipeline/{pipeline_id}/eval_dataset/{eval_dataset_id}/execute",
            ),
            json=jsonable_encoder(
                {"eval_question_ids": eval_question_ids, "params": params}
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(EvalDatasetExecution, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_eval_dataset_execution(
        self, pipeline_id: str, eval_dataset_id: str, eval_dataset_execution_id: str
    ) -> EvalDatasetExecution:
        """
        Get the status of an EvalDatasetExecution.

        Parameters:
            - pipeline_id: str.

            - eval_dataset_id: str.

            - eval_dataset_execution_id: str.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"api/pipeline/{pipeline_id}/eval_dataset/{eval_dataset_id}/execute/{eval_dataset_execution_id}",
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(EvalDatasetExecution, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_eval_dataset_execution_result(
        self, pipeline_id: str, eval_dataset_id: str
    ) -> typing.List[EvalQuestionResult]:
        """
        Get the result of an EvalDatasetExecution.

        Parameters:
            - pipeline_id: str.

            - eval_dataset_id: str.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"api/pipeline/{pipeline_id}/eval_dataset/{eval_dataset_id}/execute/result",
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.List[EvalQuestionResult], _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic

from ..core.datetime_utils import serialize_datetime


class ElasticsearchReader(pydantic.BaseModel):
    """
    Read documents from an Elasticsearch/Opensearch index.

    These documents can then be used in a downstream Llama Index data structure.

    Args:
        endpoint (str): URL (http/https) of cluster
        index (str): Name of the index (required)
        httpx_client_args (dict): Optional additional args to pass to the `httpx.Client`
    """

    is_remote: typing.Optional[bool]
    endpoint: str
    index: str
    httpx_client_args: typing.Optional[typing.Dict[str, typing.Any]]
    class_name: typing.Optional[str]

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {
            "by_alias": True,
            "exclude_unset": True,
            **kwargs,
        }
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {
            "by_alias": True,
            "exclude_unset": True,
            **kwargs,
        }
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class EtlJobNames(str, enum.Enum):
    """
    Enum for executable pipeline job names.
    """

    LOAD_DOCUMENTS_JOB = "load_documents_job"
    RUN_TRANSFORM_JOB = "run_transform_job"
    EVAL_DATASET_JOB = "eval_dataset_job"

    def visit(
        self,
        load_documents_job: typing.Callable[[], T_Result],
        run_transform_job: typing.Callable[[], T_Result],
        eval_dataset_job: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is EtlJobNames.LOAD_DOCUMENTS_JOB:
            return load_documents_job()
        if self is EtlJobNames.RUN_TRANSFORM_JOB:
            return run_transform_job()
        if self is EtlJobNames.EVAL_DATASET_JOB:
            return eval_dataset_job()



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic

from ..core.datetime_utils import serialize_datetime
from .pooling import Pooling


class HuggingFaceEmbedding(pydantic.BaseModel):
    """
    Base class for embeddings.
    """

    model_name: typing.Optional[str] = pydantic.Field(
        description="The name of the embedding model."
    )
    embed_batch_size: typing.Optional[int] = pydantic.Field(
        description="The batch size for embedding calls."
    )
    callback_manager: typing.Optional[typing.Dict[str, typing.Any]]
    tokenizer_name: str = pydantic.Field(description="Tokenizer name from HuggingFace.")
    max_length: typing.Optional[int] = pydantic.Field(
        description="Maximum length of input."
    )
    pooling: typing.Optional[Pooling] = pydantic.Field(description="Pooling strategy.")
    normalize: typing.Optional[str] = pydantic.Field(
        description="Normalize embeddings or not."
    )
    query_instruction: typing.Optional[str] = pydantic.Field(
        description="Instruction to prepend to query text."
    )
    text_instruction: typing.Optional[str] = pydantic.Field(
        description="Instruction to prepend to text."
    )
    cache_folder: typing.Optional[str] = pydantic.Field(
        description="Cache folder for huggingface files."
    )
    class_name: typing.Optional[str]

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {
            "by_alias": True,
            "exclude_unset": True,
            **kwargs,
        }
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {
            "by_alias": True,
            "exclude_unset": True,
            **kwargs,
        }
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import typing

from .chroma_vector_store import ChromaVectorStore
from .pg_vector_store import PgVectorStore
from .pinecone_vector_store import PineconeVectorStore
from .qdrant_vector_store import QdrantVectorStore
from .weaviate_vector_store import WeaviateVectorStore

DataSinkComponentOne = typing.Union[
    ChromaVectorStore,
    PineconeVectorStore,
    PgVectorStore,
    QdrantVectorStore,
    WeaviateVectorStore,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import httpx

from .core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .resources.api_key.client import ApiKeyClient, AsyncApiKeyClient
from .resources.component_definition.client import (
    AsyncComponentDefinitionClient,
    ComponentDefinitionClient,
)
from .resources.data_sink.client import AsyncDataSinkClient, DataSinkClient
from .resources.data_source.client import AsyncDataSourceClient, DataSourceClient
from .resources.eval.client import AsyncEvalClient, EvalClient
from .resources.health.client import AsyncHealthClient, HealthClient
from .resources.pipeline.client import AsyncPipelineClient, PipelineClient
from .resources.project.client import AsyncProjectClient, ProjectClient


class PlatformApi:
    def __init__(
        self,
        *,
        base_url: str,
        token: typing.Optional[typing.Union[str, typing.Callable[[], str]]] = None,
        timeout: typing.Optional[float] = 60
    ):
        self._client_wrapper = SyncClientWrapper(
            base_url=base_url, token=token, httpx_client=httpx.Client(timeout=timeout)
        )
        self.health = HealthClient(client_wrapper=self._client_wrapper)
        self.api_key = ApiKeyClient(client_wrapper=self._client_wrapper)
        self.data_sink = DataSinkClient(client_wrapper=self._client_wrapper)
        self.data_source = DataSourceClient(client_wrapper=self._client_wrapper)
        self.project = ProjectClient(client_wrapper=self._client_wrapper)
        self.pipeline = PipelineClient(client_wrapper=self._client_wrapper)
        self.eval = EvalClient(client_wrapper=self._client_wrapper)
        self.component_definition = ComponentDefinitionClient(
            client_wrapper=self._client_wrapper
        )


class AsyncPlatformApi:
    def __init__(
        self,
        *,
        base_url: str,
        token: typing.Optional[typing.Union[str, typing.Callable[[], str]]] = None,
        timeout: typing.Optional[float] = 60
    ):
        self._client_wrapper = AsyncClientWrapper(
            base_url=base_url,
            token=token,
            httpx_client=httpx.AsyncClient(timeout=timeout),
        )
        self.health = AsyncHealthClient(client_wrapper=self._client_wrapper)
        self.api_key = AsyncApiKeyClient(client_wrapper=self._client_wrapper)
        self.data_sink = AsyncDataSinkClient(client_wrapper=self._client_wrapper)
        self.data_source = AsyncDataSourceClient(client_wrapper=self._client_wrapper)
        self.project = AsyncProjectClient(client_wrapper=self._client_wrapper)
        self.pipeline = AsyncPipelineClient(client_wrapper=self._client_wrapper)
        self.eval = AsyncEvalClient(client_wrapper=self._client_wrapper)
        self.component_definition = AsyncComponentDefinitionClient(
            client_wrapper=self._client_wrapper
        )



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic

from ..core.datetime_utils import serialize_datetime


class BeautifulSoupWebReader(pydantic.BaseModel):
    """
    BeautifulSoup web page reader.

    Reads pages from the web.
    Requires the `bs4` and `urllib` packages.

    Args:
        website_extractor (Optional[Dict[str, Callable]]): A mapping of website
            hostname (e.g. google.com) to a function that specifies how to
            extract text from the BeautifulSoup obj. See DEFAULT_WEBSITE_EXTRACTOR.
    """

    is_remote: typing.Optional[bool]
    class_name: typing.Optional[str]

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {
            "by_alias": True,
            "exclude_unset": True,
            **kwargs,
        }
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {
            "by_alias": True,
            "exclude_unset": True,
            **kwargs,
        }
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from .types import (
    ApiKey,
    BasePromptTemplate,
    BasePydanticReader,
    BeautifulSoupWebReader,
    ChromaVectorStore,
    CodeSplitter,
    ConfigurableDataSinkNames,
    ConfigurableDataSourceNames,
    ConfigurableTransformationDefinition,
    ConfigurableTransformationNames,
    ConfiguredTransformationExecution,
    ConfiguredTransformationExecutionRunConfigPerOpValue,
    ConfiguredTransformationItem,
    ConfiguredTransformationItemComponent,
    ConfiguredTransformationItemComponentOne,
    DataSink,
    DataSinkComponent,
    DataSinkComponentOne,
    DataSinkCreate,
    DataSinkCreateComponent,
    DataSinkCreateComponentOne,
    DataSinkDefinition,
    DataSinkUpdateComponent,
    DataSinkUpdateComponentOne,
    DataSource,
    DataSourceComponent,
    DataSourceComponentOne,
    DataSourceCreate,
    DataSourceCreateComponent,
    DataSourceCreateComponentOne,
    DataSourceDefinition,
    DataSourceLoadExecution,
    DataSourceLoadExecutionRunConfigPerOpValue,
    DataSourceUpdateComponent,
    DataSourceUpdateComponentOne,
    DiscordReader,
    Document,
    DocumentRelationshipsValue,
    ElasticsearchReader,
    EntityExtractor,
    EtlJobNames,
    EvalDataset,
    EvalDatasetExecution,
    EvalDatasetExecutionConfig,
    EvalDatasetExecutionRunConfigPerOpValue,
    EvalExecutionParams,
    EvalLlmModelData,
    EvalQuestion,
    EvalQuestionCreate,
    EvalQuestionResult,
    GoogleDocsReader,
    GoogleSheetsReader,
    HierarchicalNodeParser,
    HtmlNodeParser,
    HttpValidationError,
    HuggingFaceEmbedding,
    JsonNodeParser,
    KeywordExtractor,
    LlmPredictor,
    MarkdownNodeParser,
    MarvinMetadataExtractor,
    MetadataMode,
    NodeParser,
    NotionPageReader,
    ObjectType,
    OpenAiEmbedding,
    PgVectorStore,
    PineconeVectorStore,
    Pipeline,
    PipelineCreate,
    Pooling,
    Project,
    ProjectCreate,
    PydanticProgramMode,
    QdrantVectorStore,
    QuestionsAnsweredExtractor,
    ReaderConfig,
    RelatedNodeInfo,
    RssReader,
    RunTransformAssetConfig,
    SentenceSplitter,
    SentenceWindowNodeParser,
    SimpleFileNodeParser,
    SimpleWebPageReader,
    SlackReader,
    StatusEnum,
    SummaryExtractor,
    SupportedEvalLlmModel,
    SupportedEvalLlmModelNames,
    TextNode,
    TextNodeRelationshipsValue,
    TitleExtractor,
    TokenTextSplitter,
    TrafilaturaWebReader,
    TransformationCategoryNames,
    TwitterTweetReader,
    ValidationError,
    ValidationErrorLocItem,
    WeaviateVectorStore,
    WikipediaReader,
    YoutubeTranscriptReader,
)
from .errors import UnprocessableEntityError
from .resources import (
    api_key,
    component_definition,
    data_sink,
    data_source,
    eval,
    health,
    pipeline,
    project,
)

__all__ = [
    "ApiKey",
    "BasePromptTemplate",
    "BasePydanticReader",
    "BeautifulSoupWebReader",
    "ChromaVectorStore",
    "CodeSplitter",
    "ConfigurableDataSinkNames",
    "ConfigurableDataSourceNames",
    "ConfigurableTransformationDefinition",
    "ConfigurableTransformationNames",
    "ConfiguredTransformationExecution",
    "ConfiguredTransformationExecutionRunConfigPerOpValue",
    "ConfiguredTransformationItem",
    "ConfiguredTransformationItemComponent",
    "ConfiguredTransformationItemComponentOne",
    "DataSink",
    "DataSinkComponent",
    "DataSinkComponentOne",
    "DataSinkCreate",
    "DataSinkCreateComponent",
    "DataSinkCreateComponentOne",
    "DataSinkDefinition",
    "DataSinkUpdateComponent",
    "DataSinkUpdateComponentOne",
    "DataSource",
    "DataSourceComponent",
    "DataSourceComponentOne",
    "DataSourceCreate",
    "DataSourceCreateComponent",
    "DataSourceCreateComponentOne",
    "DataSourceDefinition",
    "DataSourceLoadExecution",
    "DataSourceLoadExecutionRunConfigPerOpValue",
    "DataSourceUpdateComponent",
    "DataSourceUpdateComponentOne",
    "DiscordReader",
    "Document",
    "DocumentRelationshipsValue",
    "ElasticsearchReader",
    "EntityExtractor",
    "EtlJobNames",
    "EvalDataset",
    "EvalDatasetExecution",
    "EvalDatasetExecutionConfig",
    "EvalDatasetExecutionRunConfigPerOpValue",
    "EvalExecutionParams",
    "EvalLlmModelData",
    "EvalQuestion",
    "EvalQuestionCreate",
    "EvalQuestionResult",
    "GoogleDocsReader",
    "GoogleSheetsReader",
    "HierarchicalNodeParser",
    "HtmlNodeParser",
    "HttpValidationError",
    "HuggingFaceEmbedding",
    "JsonNodeParser",
    "KeywordExtractor",
    "LlmPredictor",
    "MarkdownNodeParser",
    "MarvinMetadataExtractor",
    "MetadataMode",
    "NodeParser",
    "NotionPageReader",
    "ObjectType",
    "OpenAiEmbedding",
    "PgVectorStore",
    "PineconeVectorStore",
    "Pipeline",
    "PipelineCreate",
    "Pooling",
    "Project",
    "ProjectCreate",
    "PydanticProgramMode",
    "QdrantVectorStore",
    "QuestionsAnsweredExtractor",
    "ReaderConfig",
    "RelatedNodeInfo",
    "RssReader",
    "RunTransformAssetConfig",
    "SentenceSplitter",
    "SentenceWindowNodeParser",
    "SimpleFileNodeParser",
    "SimpleWebPageReader",
    "SlackReader",
    "StatusEnum",
    "SummaryExtractor",
    "SupportedEvalLlmModel",
    "SupportedEvalLlmModelNames",
    "TextNode",
    "TextNodeRelationshipsValue",
    "TitleExtractor",
    "TokenTextSplitter",
    "TrafilaturaWebReader",
    "TransformationCategoryNames",
    "TwitterTweetReader",
    "UnprocessableEntityError",
    "ValidationError",
    "ValidationErrorLocItem",
    "WeaviateVectorStore",
    "WikipediaReader",
    "YoutubeTranscriptReader",
    "api_key",
    "component_definition",
    "data_sink",
    "data_source",
    "eval",
    "health",
    "pipeline",
    "project",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic

from ..core.datetime_utils import serialize_datetime
from .configurable_transformation_names import ConfigurableTransformationNames
from .configured_transformation_item_component import (
    ConfiguredTransformationItemComponent,
)


class ConfiguredTransformationItem(pydantic.BaseModel):
    """
    Configured transformations for pipelines.

    Similar to ConfigurableTransformation but includes a few
    more fields that are useful to the platform.
    """

    id: typing.Optional[str]
    configurable_transformation_type: ConfigurableTransformationNames = pydantic.Field(
        description="Name for the type of transformation this is (e.g. SIMPLE_NODE_PARSER). Can also be an enum instance of llama_index.ingestion.transformations.ConfigurableTransformations. This will be converted to ConfigurableTransformationNames."
    )
    component: ConfiguredTransformationItemComponent = pydantic.Field(
        description="Component that implements the transformation"
    )

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {
            "by_alias": True,
            "exclude_unset": True,
            **kwargs,
        }
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {
            "by_alias": True,
            "exclude_unset": True,
            **kwargs,
        }
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic

from ..core.datetime_utils import serialize_datetime
from .configured_transformation_item import ConfiguredTransformationItem
from .data_sink_create import DataSinkCreate
from .data_source_create import DataSourceCreate


class PipelineCreate(pydantic.BaseModel):
    """
    Schema for creating a pipeline.
    """

    configured_transformations: typing.Optional[
        typing.List[ConfiguredTransformationItem]
    ] = pydantic.Field(description="List of configured transformations.")
    data_source_ids: typing.Optional[typing.List[str]] = pydantic.Field(
        description="List of data source IDs. When provided instead of data_sources, the data sources will be looked up by ID."
    )
    data_sources: typing.Optional[typing.List[DataSourceCreate]] = pydantic.Field(
        description="List of data sources. When provided instead of data_source_ids, the data sources will be created."
    )
    data_sink_ids: typing.Optional[typing.List[str]] = pydantic.Field(
        description="List of data sink IDs. When provided instead of data_sinks, the data sinks will be looked up by ID."
    )
    data_sinks: typing.Optional[typing.List[DataSinkCreate]] = pydantic.Field(
        description="List of data sinks. When provided instead of data_sink_ids, the data sinks will be created."
    )
    name: str

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {
            "by_alias": True,
            "exclude_unset": True,
            **kwargs,
        }
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {
            "by_alias": True,
            "exclude_unset": True,
            **kwargs,
        }
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class TransformationCategoryNames(str, enum.Enum):
    """
    An enumeration.
    """

    METADATA_EXTRACTOR = "METADATA_EXTRACTOR"
    NODE_PARSER = "NODE_PARSER"
    EMBEDDING = "EMBEDDING"

    def visit(
        self,
        metadata_extractor: typing.Callable[[], T_Result],
        node_parser: typing.Callable[[], T_Result],
        embedding: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is TransformationCategoryNames.METADATA_EXTRACTOR:
            return metadata_extractor()
        if self is TransformationCategoryNames.NODE_PARSER:
            return node_parser()
        if self is TransformationCategoryNames.EMBEDDING:
            return embedding()



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic

from ..core.datetime_utils import serialize_datetime


class WeaviateVectorStore(pydantic.BaseModel):
    """
    Weaviate vector store.

    In this vector store, embeddings and docs are stored within a
    Weaviate collection.

    During query time, the index uses Weaviate to query for the top
    k most similar nodes.

    Args:
        weaviate_client (weaviate.Client): WeaviateClient
            instance from `weaviate-client` package
        index_name (Optional[str]): name for Weaviate classes
    """

    stores_text: typing.Optional[bool]
    is_embedding_query: typing.Optional[bool]
    index_name: str
    url: typing.Optional[str]
    text_key: str
    auth_config: typing.Optional[typing.Dict[str, typing.Any]]
    client_kwargs: typing.Optional[typing.Dict[str, typing.Any]]
    class_name: typing.Optional[str]

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {
            "by_alias": True,
            "exclude_unset": True,
            **kwargs,
        }
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {
            "by_alias": True,
            "exclude_unset": True,
            **kwargs,
        }
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic

from ..core.datetime_utils import serialize_datetime


class TokenTextSplitter(pydantic.BaseModel):
    """
    Implementation of splitting text that looks at word tokens.
    """

    include_metadata: typing.Optional[bool] = pydantic.Field(
        description="Whether or not to consider metadata when splitting."
    )
    include_prev_next_rel: typing.Optional[bool] = pydantic.Field(
        description="Include prev/next node relationships."
    )
    callback_manager: typing.Optional[typing.Dict[str, typing.Any]]
    chunk_size: typing.Optional[int] = pydantic.Field(
        description="The token chunk size for each chunk."
    )
    chunk_overlap: typing.Optional[int] = pydantic.Field(
        description="The token overlap of each chunk when splitting."
    )
    separator: typing.Optional[str] = pydantic.Field(
        description="Default separator for splitting into words"
    )
    backup_separators: typing.Optional[typing.List[typing.Any]] = pydantic.Field(
        description="Additional separators for splitting."
    )
    class_name: typing.Optional[str]

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {
            "by_alias": True,
            "exclude_unset": True,
            **kwargs,
        }
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {
            "by_alias": True,
            "exclude_unset": True,
            **kwargs,
        }
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import typing

from .related_node_info import RelatedNodeInfo

TextNodeRelationshipsValue = typing.Union[RelatedNodeInfo, typing.List[RelatedNodeInfo]]



# This file was auto-generated by Fern from our API Definition.

import typing

from .data_sink_update_component_one import DataSinkUpdateComponentOne

DataSinkUpdateComponent = typing.Union[
    typing.Dict[str, typing.Any], DataSinkUpdateComponentOne
]



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class PydanticProgramMode(str, enum.Enum):
    """
    Pydantic program mode.
    """

    DEFAULT = "default"
    OPENAI = "openai"
    LLM = "llm"
    GUIDANCE = "guidance"
    LM_FORMAT_ENFORCER = "lm-format-enforcer"

    def visit(
        self,
        default: typing.Callable[[], T_Result],
        openai: typing.Callable[[], T_Result],
        llm: typing.Callable[[], T_Result],
        guidance: typing.Callable[[], T_Result],
        lm_format_enforcer: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is PydanticProgramMode.DEFAULT:
            return default()
        if self is PydanticProgramMode.OPENAI:
            return openai()
        if self is PydanticProgramMode.LLM:
            return llm()
        if self is PydanticProgramMode.GUIDANCE:
            return guidance()
        if self is PydanticProgramMode.LM_FORMAT_ENFORCER:
            return lm_format_enforcer()



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic

from ..core.datetime_utils import serialize_datetime
from .base_prompt_template import BasePromptTemplate
from .pydantic_program_mode import PydanticProgramMode


class LlmPredictor(pydantic.BaseModel):
    """
    LLM predictor class.

    A lightweight wrapper on top of LLMs that handles:
    - conversion of prompts to the string input format expected by LLMs
    - logging of prompts and responses to a callback manager

    NOTE: Mostly keeping around for legacy reasons. A potential future path is to
    deprecate this class and move all functionality into the LLM class.
    """

    system_prompt: typing.Optional[str]
    query_wrapper_prompt: typing.Optional[BasePromptTemplate]
    pydantic_program_mode: typing.Optional[PydanticProgramMode]
    class_name: typing.Optional[str]

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {
            "by_alias": True,
            "exclude_unset": True,
            **kwargs,
        }
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {
            "by_alias": True,
            "exclude_unset": True,
            **kwargs,
        }
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic

from ..core.datetime_utils import serialize_datetime
from .metadata_mode import MetadataMode


class EntityExtractor(pydantic.BaseModel):
    """
    Entity extractor. Extracts `entities` into a metadata field using a default model
    `tomaarsen/span-marker-mbert-base-multinerd` and the SpanMarker library.

    Install SpanMarker with `pip install span-marker`.
    """

    is_text_node_only: typing.Optional[bool]
    show_progress: typing.Optional[bool] = pydantic.Field(
        description="Whether to show progress."
    )
    metadata_mode: typing.Optional[MetadataMode] = pydantic.Field(
        description="Metadata mode to use when reading nodes."
    )
    node_text_template: typing.Optional[str] = pydantic.Field(
        description="Template to represent how node text is mixed with metadata text."
    )
    disable_template_rewrite: typing.Optional[bool] = pydantic.Field(
        description="Disable the node template rewrite."
    )
    in_place: typing.Optional[bool] = pydantic.Field(
        description="Whether to process nodes in place."
    )
    model_name: typing.Optional[str] = pydantic.Field(
        description="The model name of the SpanMarker model to use."
    )
    prediction_threshold: typing.Optional[float] = pydantic.Field(
        description="The confidence threshold for accepting predictions."
    )
    span_joiner: typing.Optional[str] = pydantic.Field(
        description="The separator between entity names."
    )
    label_entities: typing.Optional[bool] = pydantic.Field(
        description="Include entity class labels or not."
    )
    device: typing.Optional[str] = pydantic.Field(
        description="Device to run model on, i.e. 'cuda', 'cpu'"
    )
    entity_map: typing.Optional[typing.Dict[str, str]] = pydantic.Field(
        description="Mapping of entity class names to usable names."
    )
    class_name: typing.Optional[str]

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {
            "by_alias": True,
            "exclude_unset": True,
            **kwargs,
        }
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {
            "by_alias": True,
            "exclude_unset": True,
            **kwargs,
        }
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic

from ..core.datetime_utils import serialize_datetime
from .configured_transformation_execution_run_config_per_op_value import (
    ConfiguredTransformationExecutionRunConfigPerOpValue,
)
from .etl_job_names import EtlJobNames
from .status_enum import StatusEnum


class ConfiguredTransformationExecution(pydantic.BaseModel):
    """
    Schema for a job that executes a transform step in a pipeline.
    """

    id: typing.Optional[str] = pydantic.Field(description="Unique identifier")
    created_at: typing.Optional[dt.datetime] = pydantic.Field(
        description="Creation datetime"
    )
    updated_at: typing.Optional[dt.datetime] = pydantic.Field(
        description="Update datetime"
    )
    job_name: EtlJobNames
    status: StatusEnum
    started_at: typing.Optional[dt.datetime]
    ended_at: typing.Optional[dt.datetime]
    run_config_per_op: typing.Dict[
        str, ConfiguredTransformationExecutionRunConfigPerOpValue
    ] = pydantic.Field(description="Run config that was given to the Dagster job")
    partitions: typing.Optional[typing.Dict[str, str]] = pydantic.Field(
        description="Partition information"
    )
    pipeline_id: str = pydantic.Field(
        description="The ID for the Pipeline for which this execution ran a particular transformation."
    )
    configured_transformation_id: str = pydantic.Field(
        description="The ID for the specific configured transformation step this execution ran."
    )

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {
            "by_alias": True,
            "exclude_unset": True,
            **kwargs,
        }
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {
            "by_alias": True,
            "exclude_unset": True,
            **kwargs,
        }
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ConfigurableDataSinkNames(str, enum.Enum):
    """
    An enumeration.
    """

    CHROMA = "CHROMA"
    PINECONE = "PINECONE"
    POSTGRES = "POSTGRES"
    QDRANT = "QDRANT"
    WEAVIATE = "WEAVIATE"

    def visit(
        self,
        chroma: typing.Callable[[], T_Result],
        pinecone: typing.Callable[[], T_Result],
        postgres: typing.Callable[[], T_Result],
        qdrant: typing.Callable[[], T_Result],
        weaviate: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is ConfigurableDataSinkNames.CHROMA:
            return chroma()
        if self is ConfigurableDataSinkNames.PINECONE:
            return pinecone()
        if self is ConfigurableDataSinkNames.POSTGRES:
            return postgres()
        if self is ConfigurableDataSinkNames.QDRANT:
            return qdrant()
        if self is ConfigurableDataSinkNames.WEAVIATE:
            return weaviate()



# This file was auto-generated by Fern from our API Definition.

import typing

from .data_sink_component_one import DataSinkComponentOne

DataSinkComponent = typing.Union[typing.Dict[str, typing.Any], DataSinkComponentOne]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic

from ..core.datetime_utils import serialize_datetime


class SentenceSplitter(pydantic.BaseModel):
    """
    Parse text with a preference for complete sentences.

    In general, this class tries to keep sentences and paragraphs together. Therefore
    compared to the original TokenTextSplitter, there are less likely to be
    hanging sentences or parts of sentences at the end of the node chunk.
    """

    include_metadata: typing.Optional[bool] = pydantic.Field(
        description="Whether or not to consider metadata when splitting."
    )
    include_prev_next_rel: typing.Optional[bool] = pydantic.Field(
        description="Include prev/next node relationships."
    )
    callback_manager: typing.Optional[typing.Dict[str, typing.Any]]
    chunk_size: typing.Optional[int] = pydantic.Field(
        description="The token chunk size for each chunk."
    )
    chunk_overlap: typing.Optional[int] = pydantic.Field(
        description="The token overlap of each chunk when splitting."
    )
    separator: typing.Optional[str] = pydantic.Field(
        description="Default separator for splitting into words"
    )
    paragraph_separator: typing.Optional[str] = pydantic.Field(
        description="Separator between paragraphs."
    )
    secondary_chunking_regex: typing.Optional[str] = pydantic.Field(
        description="Backup regex for splitting into sentences."
    )
    class_name: typing.Optional[str]

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {
            "by_alias": True,
            "exclude_unset": True,
            **kwargs,
        }
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {
            "by_alias": True,
            "exclude_unset": True,
            **kwargs,
        }
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

import pydantic

from llama_index_client.core.api_error import ApiError
from llama_index_client.core.client_wrapper import (
    AsyncClientWrapper,
    SyncClientWrapper,
)
from llama_index_client.core.jsonable_encoder import jsonable_encoder
from llama_index_client.errors.unprocessable_entity_error import (
    UnprocessableEntityError,
)
from llama_index_client.types.configurable_data_sink_names import (
    ConfigurableDataSinkNames,
)
from llama_index_client.types.data_sink import DataSink
from llama_index_client.types.data_sink_create import DataSinkCreate
from llama_index_client.types.data_sink_update_component import (
    DataSinkUpdateComponent,
)
from llama_index_client.types.http_validation_error import HttpValidationError

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class DataSinkClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create_data_sink(self, *, request: DataSinkCreate) -> DataSink:
        """
        Create a new data sink.

        Parameters:
            - request: DataSinkCreate.
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", "api/data_sink"
            ),
            json=jsonable_encoder(request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(DataSink, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def upsert_data_sink(self, *, request: DataSinkCreate) -> DataSink:
        """
        Upserts a data sink.
        Updates if a data sink with the same name and user_id already exists. Otherwise, creates a new data sink.

        Parameters:
            - request: DataSinkCreate.
        """
        _response = self._client_wrapper.httpx_client.request(
            "PUT",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", "api/data_sink"
            ),
            json=jsonable_encoder(request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(DataSink, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_data_sink(self, data_sink_id: str) -> DataSink:
        """
        Get a data sink by ID.

        Parameters:
            - data_sink_id: str.
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"api/data_sink/{data_sink_id}",
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(DataSink, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update_data_sink(
        self,
        data_sink_id: str,
        *,
        sink_type: ConfigurableDataSinkNames,
        component: typing.Optional[DataSinkUpdateComponent] = OMIT,
        name: typing.Optional[str] = OMIT,
    ) -> DataSink:
        """
        Update a data sink by ID.

        Parameters:
            - data_sink_id: str.

            - sink_type: ConfigurableDataSinkNames.

            - component: typing.Optional[DataSinkUpdateComponent].

            - name: typing.Optional[str].
        """
        _request: typing.Dict[str, typing.Any] = {"sink_type": sink_type}
        if component is not OMIT:
            _request["component"] = component
        if name is not OMIT:
            _request["name"] = name
        _response = self._client_wrapper.httpx_client.request(
            "PUT",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"api/data_sink/{data_sink_id}",
            ),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(DataSink, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete_data_sink(self, data_sink_id: str) -> None:
        """
        Delete a data sink by ID.

        Parameters:
            - data_sink_id: str.
        """
        _response = self._client_wrapper.httpx_client.request(
            "DELETE",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"api/data_sink/{data_sink_id}",
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncDataSinkClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create_data_sink(self, *, request: DataSinkCreate) -> DataSink:
        """
        Create a new data sink.

        Parameters:
            - request: DataSinkCreate.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", "api/data_sink"
            ),
            json=jsonable_encoder(request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(DataSink, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def upsert_data_sink(self, *, request: DataSinkCreate) -> DataSink:
        """
        Upserts a data sink.
        Updates if a data sink with the same name and user_id already exists. Otherwise, creates a new data sink.

        Parameters:
            - request: DataSinkCreate.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "PUT",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", "api/data_sink"
            ),
            json=jsonable_encoder(request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(DataSink, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_data_sink(self, data_sink_id: str) -> DataSink:
        """
        Get a data sink by ID.

        Parameters:
            - data_sink_id: str.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"api/data_sink/{data_sink_id}",
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(DataSink, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update_data_sink(
        self,
        data_sink_id: str,
        *,
        sink_type: ConfigurableDataSinkNames,
        component: typing.Optional[DataSinkUpdateComponent] = OMIT,
        name: typing.Optional[str] = OMIT,
    ) -> DataSink:
        """
        Update a data sink by ID.

        Parameters:
            - data_sink_id: str.

            - sink_type: ConfigurableDataSinkNames.

            - component: typing.Optional[DataSinkUpdateComponent].

            - name: typing.Optional[str].
        """
        _request: typing.Dict[str, typing.Any] = {"sink_type": sink_type}
        if component is not OMIT:
            _request["component"] = component
        if name is not OMIT:
            _request["name"] = name
        _response = await self._client_wrapper.httpx_client.request(
            "PUT",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"api/data_sink/{data_sink_id}",
            ),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(DataSink, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete_data_sink(self, data_sink_id: str) -> None:
        """
        Delete a data sink by ID.

        Parameters:
            - data_sink_id: str.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "DELETE",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"api/data_sink/{data_sink_id}",
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

import pydantic

from llama_index_client.core.api_error import ApiError
from llama_index_client.core.client_wrapper import (
    AsyncClientWrapper,
    SyncClientWrapper,
)
from llama_index_client.core.jsonable_encoder import jsonable_encoder
from llama_index_client.core.remove_none_from_dict import (
    remove_none_from_dict,
)
from llama_index_client.errors.unprocessable_entity_error import (
    UnprocessableEntityError,
)
from llama_index_client.types.eval_dataset import EvalDataset
from llama_index_client.types.http_validation_error import HttpValidationError
from llama_index_client.types.pipeline import Pipeline
from llama_index_client.types.pipeline_create import PipelineCreate
from llama_index_client.types.project import Project
from llama_index_client.types.project_create import ProjectCreate

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ProjectClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list_projects(
        self, *, project_name: typing.Optional[str] = None
    ) -> typing.List[Project]:
        """
        List projects or get one by name

        Parameters:
            - project_name: typing.Optional[str].
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", "api/project"
            ),
            params=remove_none_from_dict({"project_name": project_name}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.List[Project], _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create_project(self, *, request: ProjectCreate) -> Project:
        """
        Create a new project.

        Parameters:
            - request: ProjectCreate.
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", "api/project"
            ),
            json=jsonable_encoder(request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Project, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def upsert_project(self, *, request: ProjectCreate) -> Project:
        """
        Upsert a project.
        Updates if a project with the same name already exists. Otherwise, creates a new project.

        Parameters:
            - request: ProjectCreate.
        """
        _response = self._client_wrapper.httpx_client.request(
            "PUT",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", "api/project"
            ),
            json=jsonable_encoder(request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Project, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_project(self, project_id: str) -> Project:
        """
        Get a project by ID.

        Parameters:
            - project_id: str.
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"api/project/{project_id}"
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Project, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update_existing_project(self, project_id: str, *, name: str) -> Project:
        """
        Update an existing project.

        Parameters:
            - project_id: str.

            - name: str.
        """
        _response = self._client_wrapper.httpx_client.request(
            "PUT",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"api/project/{project_id}"
            ),
            json=jsonable_encoder({"name": name}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Project, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create_pipeline_for_project(
        self, project_id: str, *, request: PipelineCreate
    ) -> Pipeline:
        """
        Create a new pipeline for a project.

        Parameters:
            - project_id: str.

            - request: PipelineCreate.
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"api/project/{project_id}/pipeline",
            ),
            json=jsonable_encoder(request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Pipeline, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def upsert_pipeline_for_project(
        self, project_id: str, *, request: PipelineCreate
    ) -> Pipeline:
        """
        Upsert a pipeline for a project.
        Updates if a pipeline with the same name and project_id already exists. Otherwise, creates a new pipeline.

        Parameters:
            - project_id: str.

            - request: PipelineCreate.
        """
        _response = self._client_wrapper.httpx_client.request(
            "PUT",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"api/project/{project_id}/pipeline",
            ),
            json=jsonable_encoder(request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Pipeline, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_datasets_for_project(self, project_id: str) -> typing.List[EvalDataset]:
        """
        Get all eval datasets for a project.

        Parameters:
            - project_id: str.
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"api/project/{project_id}/eval/dataset",
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.List[EvalDataset], _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create_eval_dataset_for_project(
        self, project_id: str, *, name: str
    ) -> EvalDataset:
        """
        Create a new eval dataset for a project.

        Parameters:
            - project_id: str.

            - name: str.
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"api/project/{project_id}/eval/dataset",
            ),
            json=jsonable_encoder({"name": name}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(EvalDataset, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncProjectClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list_projects(
        self, *, project_name: typing.Optional[str] = None
    ) -> typing.List[Project]:
        """
        List projects or get one by name

        Parameters:
            - project_name: typing.Optional[str].
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", "api/project"
            ),
            params=remove_none_from_dict({"project_name": project_name}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.List[Project], _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create_project(self, *, request: ProjectCreate) -> Project:
        """
        Create a new project.

        Parameters:
            - request: ProjectCreate.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", "api/project"
            ),
            json=jsonable_encoder(request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Project, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def upsert_project(self, *, request: ProjectCreate) -> Project:
        """
        Upsert a project.
        Updates if a project with the same name already exists. Otherwise, creates a new project.

        Parameters:
            - request: ProjectCreate.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "PUT",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", "api/project"
            ),
            json=jsonable_encoder(request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Project, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_project(self, project_id: str) -> Project:
        """
        Get a project by ID.

        Parameters:
            - project_id: str.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"api/project/{project_id}"
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Project, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update_existing_project(self, project_id: str, *, name: str) -> Project:
        """
        Update an existing project.

        Parameters:
            - project_id: str.

            - name: str.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "PUT",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"api/project/{project_id}"
            ),
            json=jsonable_encoder({"name": name}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Project, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create_pipeline_for_project(
        self, project_id: str, *, request: PipelineCreate
    ) -> Pipeline:
        """
        Create a new pipeline for a project.

        Parameters:
            - project_id: str.

            - request: PipelineCreate.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"api/project/{project_id}/pipeline",
            ),
            json=jsonable_encoder(request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Pipeline, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def upsert_pipeline_for_project(
        self, project_id: str, *, request: PipelineCreate
    ) -> Pipeline:
        """
        Upsert a pipeline for a project.
        Updates if a pipeline with the same name and project_id already exists. Otherwise, creates a new pipeline.

        Parameters:
            - project_id: str.

            - request: PipelineCreate.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "PUT",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"api/project/{project_id}/pipeline",
            ),
            json=jsonable_encoder(request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Pipeline, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_datasets_for_project(
        self, project_id: str
    ) -> typing.List[EvalDataset]:
        """
        Get all eval datasets for a project.

        Parameters:
            - project_id: str.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"api/project/{project_id}/eval/dataset",
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.List[EvalDataset], _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create_eval_dataset_for_project(
        self, project_id: str, *, name: str
    ) -> EvalDataset:
        """
        Create a new eval dataset for a project.

        Parameters:
            - project_id: str.

            - name: str.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"api/project/{project_id}/eval/dataset",
            ),
            json=jsonable_encoder({"name": name}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(EvalDataset, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(HttpValidationError, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic

from ..core.datetime_utils import serialize_datetime
from .metadata_mode import MetadataMode


class MarvinMetadataExtractor(pydantic.BaseModel):
    """
    Metadata extractor.
    """

    is_text_node_only: typing.Optional[bool]
    show_progress: typing.Optional[bool] = pydantic.Field(
        description="Whether to show progress."
    )
    metadata_mode: typing.Optional[MetadataMode] = pydantic.Field(
        description="Metadata mode to use when reading nodes."
    )
    node_text_template: typing.Optional[str] = pydantic.Field(
        description="Template to represent how node text is mixed with metadata text."
    )
    disable_template_rewrite: typing.Optional[bool] = pydantic.Field(
        description="Disable the node template rewrite."
    )
    in_place: typing.Optional[bool] = pydantic.Field(
        description="Whether to process nodes in place."
    )
    marvin_model: typing.Any
    llm_model_string: typing.Optional[str] = pydantic.Field(
        description="The LLM model string to use for extracting custom metadata"
    )
    class_name: typing.Optional[str]

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {
            "by_alias": True,
            "exclude_unset": True,
            **kwargs,
        }
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {
            "by_alias": True,
            "exclude_unset": True,
            **kwargs,
        }
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class SupportedEvalLlmModelNames(str, enum.Enum):
    """
    An enumeration.
    """

    GPT_3_5_TURBO = "GPT_3_5_TURBO"
    GPT_4 = "GPT_4"

    def visit(
        self,
        gpt_3_5_turbo: typing.Callable[[], T_Result],
        gpt_4: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is SupportedEvalLlmModelNames.GPT_3_5_TURBO:
            return gpt_3_5_turbo()
        if self is SupportedEvalLlmModelNames.GPT_4:
            return gpt_4()



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic

from ..core.datetime_utils import serialize_datetime
from .base_pydantic_reader import BasePydanticReader


class ReaderConfig(pydantic.BaseModel):
    """
    Represents a reader and it's input arguments.
    """

    reader: BasePydanticReader = pydantic.Field(description="Reader to use.")
    reader_args: typing.Optional[typing.List[typing.Any]] = pydantic.Field(
        description="Reader args."
    )
    reader_kwargs: typing.Optional[typing.Dict[str, typing.Any]] = pydantic.Field(
        description="Reader kwargs."
    )
    class_name: typing.Optional[str]

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {
            "by_alias": True,
            "exclude_unset": True,
            **kwargs,
        }
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {
            "by_alias": True,
            "exclude_unset": True,
            **kwargs,
        }
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic

from ..core.datetime_utils import serialize_datetime
from .eval_execution_params import EvalExecutionParams
from .text_node import TextNode


class EvalQuestionResult(pydantic.BaseModel):
    """
    Schema for the result of an eval question execution.
    """

    eval_question_id: str = pydantic.Field(
        description="The ID of the question that was executed."
    )
    pipeline_id: str = pydantic.Field(
        description="The ID of the pipeline that the question was executed against."
    )
    source_nodes: typing.List[TextNode] = pydantic.Field(
        description="The nodes retrieved by the pipeline for the given question."
    )
    answer: str = pydantic.Field(description="The answer to the question.")
    eval_metrics: typing.Dict[str, typing.Any] = pydantic.Field(
        description="The eval metrics for the question."
    )
    eval_dataset_execution_id: str = pydantic.Field(
        description="The ID of the EvalDatasetExecution that this result was generated from."
    )
    eval_dataset_execution_params: EvalExecutionParams = pydantic.Field(
        description="The EvalExecutionParams that were used when this result was generated."
    )
    eval_finished_at: dt.datetime = pydantic.Field(
        description="The timestamp when the eval finished."
    )
    class_name: typing.Optional[str]

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {
            "by_alias": True,
            "exclude_unset": True,
            **kwargs,
        }
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {
            "by_alias": True,
            "exclude_unset": True,
            **kwargs,
        }
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from . import (
    api_key,
    component_definition,
    data_sink,
    data_source,
    eval,
    health,
    pipeline,
    project,
)

__all__ = [
    "api_key",
    "component_definition",
    "data_sink",
    "data_source",
    "eval",
    "health",
    "pipeline",
    "project",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic

from ..core.datetime_utils import serialize_datetime


class GoogleSheetsReader(pydantic.BaseModel):
    """
    Google Sheets reader.

    Reads a sheet as TSV from Google Sheets
    """

    is_remote: typing.Optional[bool]
    class_name: typing.Optional[str]

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {
            "by_alias": True,
            "exclude_unset": True,
            **kwargs,
        }
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {
            "by_alias": True,
            "exclude_unset": True,
            **kwargs,
        }
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}
